@@ -7,7 +7,7 @@ Attean::API - Utility package for loading all Attean role packages.
 
 =head1 VERSION
 
-This document describes Attean::API version 0.007
+This document describes Attean::API version 0.012
 
 =head1 SYNOPSIS
 
@@ -19,27 +19,51 @@ This document describes Attean::API version 0.007
 This is a utility package that will load all the Attean-related Moo roles
 in the Attean::API namespace.
 
+=head1 METHODS
+
+=over 4
+
 =cut
 
-package Attean::API::ResultOrTerm 0.007 {
+package Attean::API::ResultOrTerm 0.012 {
 	use Moo::Role;
 }
 
-package Attean::API::BlankOrIRI 0.007 {
+package Attean::API::BlankOrIRI 0.012 {
 	use Moo::Role;
 }
 
-package Attean::API::TermOrVariable 0.007 {
+package Attean::API::TermOrVariable 0.012 {
 	use Moo::Role;
+	use Scalar::Util qw(blessed);
 	use Sub::Install;
 	use Sub::Util qw(set_subname);
 	use namespace::clean;
 
+	with 'Attean::API::SPARQLSerializable';
+
 	sub equals {
 		my ($a, $b)	= @_;
 		return ($a->as_string eq $b->as_string);
 	}
 	
+	sub apply_binding {
+		my $self	= shift;
+		my $class	= ref($self);
+		my $bind	= shift;
+		if ($self->does('Attean::API::Variable')) {
+			my $name	= $self->value;
+			my $replace	= $bind->value($name);
+			if (defined($replace) and blessed($replace)) {
+				return $replace;
+			} else {
+				return $self;
+			}
+		} else {
+			return $self;
+		}
+	}
+
 	BEGIN {
 		my %types	= (
 			variable	=> 'Variable',
@@ -59,30 +83,46 @@ package Attean::API::TermOrVariable 0.007 {
 	}
 }
 
-package Attean::Mapper 0.007 {
+package Attean::Mapper 0.012 {
 	use Moo::Role;
 	requires 'map'; # my $that = $object->map($this)
 }
 
-package Attean::API::Variable 0.007 {
+package Attean::API::Variable 0.012 {
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Moo::Role;
+	use namespace::clean;
+
 	with 'Attean::API::TermOrVariable';
-	sub as_sparql {
-		my $self	= shift;
-		return '?' . $self->value;
-	}
-	
+
+=item C<< as_string >>
+
+Returns a string representation of the variable.'
+
+=cut
+
 	sub as_string {
 		my $self	= shift;
 		return '?' . $self->value;
 	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $t	= AtteanX::SPARQL::Token->fast_constructor( VAR, -1, -1, -1, -1, [$self->value] );
+		return Attean::ListIterator->new( values => [$t], item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 }
 
-package Attean::API::CanonicalizingBindingSet 0.007 {
+package Attean::API::CanonicalizingBindingSet 0.012 {
 	use Moo::Role;
 	use Attean::RDF;
 	use namespace::clean;
+
+	with 'MooX::Log::Any';
 	requires 'elements';
+
 	sub canonical_set {
 		my $self	= shift;
 		my ($set)	= $self->canonical_set_with_mapping;
@@ -115,6 +155,7 @@ package Attean::API::CanonicalizingBindingSet 0.007 {
 		foreach my $i (0 .. $#tuples) {
 			my $p		= $tuples[$i];
 			my ($str, $t)	= @$p;
+			my $item_class	= ref($t);
 			my ($next, $last)	= ('')x2;
 			$last	= $tuples[$i-1][0] if ($i > 0);
 			$next	= $tuples[$i+1][0] if ($i < $#tuples);
@@ -130,8 +171,8 @@ package Attean::API::CanonicalizingBindingSet 0.007 {
 					my $type	= ($prefix eq '?' ? 'variable' : 'blank');
 					$mapping{ $key }	= { id => $id, prefix => $prefix, type => $type };
 					my %t		= $p->[1]->mapping;
-					$t{ $pos }	= ($type eq 'blank') ? blank($id) : variable($id);
-					my $t	= Attean::Triple->new( %t );
+					$t{ $pos }	= ($type eq 'blank') ? Attean::Blank->new($id) : Attean::Variable->new($id);
+					my $t	= $item_class->new( %t );
 					$p->[1]	= $t;
 					$p->[0]	= $t->tuples_string;
 				}
@@ -140,6 +181,7 @@ package Attean::API::CanonicalizingBindingSet 0.007 {
 	
 		foreach my $p (@tuples) {
 			my ($str, $t)	= @$p;
+			my $item_class	= ref($t);
 			foreach my $pos (reverse $t->variables) {
 				if (defined(my $tstr = $p->[2]{$pos})) {
 					$tstr	=~ /^([?]|_:)([^#]+)$/;
@@ -148,15 +190,15 @@ package Attean::API::CanonicalizingBindingSet 0.007 {
 					my $key		= "$prefix$name";
 					delete $p->[2]{$pos};
 					unless (exists($mapping{$key})) {
-						warn "Cannot canonicalize binding set";
+						$self->error("Cannot canonicalize binding set");
 						return;
 					}
 					my $id		= $mapping{$key}{id};
 					my $type	= ($prefix eq '?' ? 'variable' : 'blank');
 					$mapping{ $key }	= { id => $id, prefix => $prefix, type => $type };
 					my %t		= $p->[1]->mapping;
-					$t{ $pos }	= ($type eq 'blank') ? blank($id) : variable($id);
-					my $t	= Attean::Triple->new( %t );
+					$t{ $pos }	= ($type eq 'blank') ? Attean::Blank->new($id) : Attean::Variable->new($id);
+					my $t	= $item_class->new( %t );
 					$p->[1]	= $t;
 					$p->[0]	= $t->tuples_string;
 				}
@@ -169,7 +211,7 @@ package Attean::API::CanonicalizingBindingSet 0.007 {
 	}
 }
 
-package Attean::API 0.007 {
+package Attean::API 0.012 {
 	use Attean::API::Term;
 	use Attean::API::Store;
 	use Attean::API::Model;
@@ -179,6 +221,7 @@ package Attean::API 0.007 {
 	use Attean::API::Query;
 	use Attean::API::Expression;
 	use Attean::API::Plan;
+	use Attean::API::QueryPlanner;
 	
 	use Attean::Variable;
 	use Attean::Blank;
@@ -189,6 +232,8 @@ package Attean::API 0.007 {
 
 __END__
 
+=back
+
 =head1 BUGS
 
 Please report any bugs or feature requests to through the GitHub web interface
@@ -204,7 +249,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
