@@ -7,7 +7,7 @@ Attean::API::Iterator - Typed iterator
 
 =head1 VERSION
 
-This document describes Attean::API::Iterator version 0.007
+This document describes Attean::API::Iterator version 0.012
 
 =head1 DESCRIPTION
 
@@ -92,11 +92,9 @@ C<< $limit >> elements of the referent.
 Returns a new L<Attean::API::RepeatableIterator> object containing all the
 elements from the referent.
 
-=back
-
 =cut
 
-package Attean::API::Iterator 0.007 {
+package Attean::API::Iterator 0.012 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Str Object InstanceOf);
@@ -148,8 +146,10 @@ package Attean::API::Iterator 0.007 {
 			my $class	= ref($self);
 			my $term	= $self->$orig(@_);
 			return unless defined($term);
-			unless ($term->does($type)) {
-				die "${class} returned an element that failed conformance check for $type";
+			if (blessed($term)) {
+				unless ($term->does($type) or $term->isa($type)) {
+					die "${class} returned an element that failed conformance check for $type: $term";
+				}
 			}
 			return $term;
 		};
@@ -243,6 +243,12 @@ package Attean::API::Iterator 0.007 {
 		return Attean::ListIterator->new( values => \@data, item_type => $self->item_type );
 	}
 	
+=item C<< debug( [$name] ) >>
+
+Print each item as it is consumed (with the string generated by C<< as_string >>), prepended by C<< $name >>.
+
+=cut
+
 	sub debug {
 		my $self	= shift;
 		my $name	= shift // 'Iterator item';
@@ -250,7 +256,7 @@ package Attean::API::Iterator 0.007 {
 	}
 }
 
-package Attean::API::RepeatableIterator 0.007 {
+package Attean::API::RepeatableIterator 0.012 {
 	use Moo::Role;
 	requires 'reset';
 	
@@ -286,7 +292,7 @@ package Attean::API::CanonicalizingBindingIterator {
 	}
 }
 
-package Attean::API::ResultOrTermIterator 0.007 {
+package Attean::API::ResultOrTermIterator 0.012 {
 	use Moo::Role;
 	use Types::Standard qw(ArrayRef Str);
 	has 'variables' => (is => 'rw', isa => ArrayRef[Str], default => sub { [] });
@@ -303,9 +309,28 @@ package Attean::API::ResultOrTermIterator 0.007 {
 			}
 		});
 	}
+	
+	around 'grep' => sub {
+		my $orig	= shift;
+		my $self	= shift;
+		my $block	= shift;
+		my $iter	= $orig->($self, $block, @_);
+		Attean::CodeIterator->new(
+			item_type => $iter->item_type,
+			generator => sub {
+				while (1) {
+					my $item	= $iter->next();
+					return unless defined($item);
+					local($_)	= $item;
+					return $item if ($block->($item));
+				}
+			},
+			variables => $self->variables,
+		);
+	};
 }
 
-package Attean::API::TripleIterator 0.007 {
+package Attean::API::TripleIterator 0.012 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 	sub as_quads {
@@ -315,12 +340,12 @@ package Attean::API::TripleIterator 0.007 {
 	}
 }
 
-package Attean::API::QuadIterator 0.007 {
+package Attean::API::QuadIterator 0.012 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 }
 
-package Attean::API::MixedStatementIterator 0.007 {
+package Attean::API::MixedStatementIterator 0.012 {
 	use Moo::Role;
 	with 'Attean::API::CanonicalizingBindingIterator';
 	sub as_quads {
@@ -333,7 +358,7 @@ package Attean::API::MixedStatementIterator 0.007 {
 	}
 }
 
-package Attean::API::ResultIterator 0.007 {
+package Attean::API::ResultIterator 0.012 {
 	use Moo::Role;
 	use Types::Standard qw(Str ArrayRef);
 	with 'Attean::API::CanonicalizingBindingIterator';
@@ -354,7 +379,7 @@ package Attean::API::ResultIterator 0.007 {
 	}
 }
 
-package Attean::API::TermIterator 0.007 {
+package Attean::API::TermIterator 0.012 {
 	use Moo::Role;
 	sub canonicalize {
 		my $self	= shift;
@@ -368,6 +393,8 @@ package Attean::API::TermIterator 0.007 {
 
 __END__
 
+=back
+
 =head1 BUGS
 
 Please report any bugs or feature requests to through the GitHub web interface
@@ -385,7 +412,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
