@@ -7,7 +7,7 @@ Attean::API::Term - RDF Terms
 
 =head1 VERSION
 
-This document describes Attean::API::Term version 0.007
+This document describes Attean::API::Term version 0.012
 
 =head1 DESCRIPTION
 
@@ -39,11 +39,9 @@ This role provides default implementations of the following methods:
 
 Returns a string serialization of the term.
 
-=back
-
 =cut
 
-package Attean::API::Term 0.007 {
+package Attean::API::Term 0.012 {
 	use Moo::Role;
 	
 	with 'Attean::API::TermOrVariable', 'Attean::API::ResultOrTerm';
@@ -53,9 +51,13 @@ package Attean::API::Term 0.007 {
 	sub as_string {
 		shift->ntriples_string();
 	}
-	sub as_sparql {
-		shift->ntriples_string();
-	}
+
+=item C<< ebv >>
+
+Returns true if the term has a true SPARQL "effective boolean value", false otherwise.
+
+=cut
+
 	sub ebv { return 0; }
 	
 	requires 'compare';
@@ -104,14 +106,18 @@ package Attean::API::Term 0.007 {
 	}
 }
 
-package Attean::API::Literal 0.007 {
+package Attean::API::Literal 0.012 {
 	use Moo::Role;
 	use IRI;
 	use Scalar::Util qw(blessed);
 	use Types::Standard qw(Maybe Str ConsumerOf);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use Attean::API::Query;
 	use namespace::clean;
 	
 	with 'Attean::API::Term';
+	with 'Attean::API::SPARQLSerializable';
 	
 	requires 'language'; # => (is => 'ro', isa => 'Maybe[Str]', predicate => 'has_language');
 	requires 'datatype'; # => (is => 'ro', isa => 'Attean::API::IRI', required => 1, coerce => 1, default => sub { IRI->new(value => 'http://www.w3.org/2001/XMLSchema#string') });
@@ -153,6 +159,32 @@ package Attean::API::Literal 0.007 {
 		}
 	};
 	
+	sub sparql_tokens {
+		my $self	= shift;
+		my @tokens;
+		
+		my $dt = $self->datatype;
+		if ($self->does('Attean::API::NumericLiteral') and $dt->value eq 'http://www.w3.org/2001/XMLSchema#integer') {
+			if ($self->value =~ /^\d+$/) {
+				my $t	= AtteanX::SPARQL::Token->fast_constructor( INTEGER, -1, -1, -1, -1, [$self->value] );
+				return Attean::ListIterator->new( values => [$t], item_type => 'AtteanX::SPARQL::Token' );
+			}
+		}
+		
+		my $t	= AtteanX::SPARQL::Token->fast_constructor( STRING1D, -1, -1, -1, -1, [$self->value] );
+		push(@tokens, $t);
+		if (my $lang = $self->language) {
+			my $l	= AtteanX::SPARQL::Token->fast_constructor( LANG, -1, -1, -1, -1, ["$lang"] );
+			push(@tokens, $l);
+		} else {
+			if ($dt->value ne 'http://www.w3.org/2001/XMLSchema#string') {
+				push(@tokens, AtteanX::SPARQL::Token->hathat);
+				push(@tokens, $dt->sparql_tokens->elements);
+			}
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 	sub ebv {
 		my $self	= shift;
 		my $value	= $self->value;
@@ -202,6 +234,34 @@ package Attean::API::Literal 0.007 {
 		return %args;
 	}
 	
+	sub argument_compatible {
+		my $self	= shift;
+		my @terms	= @_;
+		
+		if (my $l = $self->language) {
+			foreach my $t (@terms) {
+				return 0 unless ($t->does('Attean::API::Literal'));
+				if ($t->language) {
+					return 0 unless (defined($t->language));
+					return 0 unless ($t->language eq $l);
+				} else {
+					return 0 unless ($t->datatype->value eq 'http://www.w3.org/2001/XMLSchema#string');
+				}
+			}
+			return 1;
+		} elsif ($self->datatype->value eq 'http://www.w3.org/2001/XMLSchema#string') {
+			foreach my $t (@terms) {
+				return 0 unless ($t->does('Attean::API::Literal'));
+				return 0 if ($t->language);
+				return 0 unless (blessed($t->datatype));
+				return 0 unless ($t->datatype->value eq 'http://www.w3.org/2001/XMLSchema#string');
+			}
+			return 1;
+		}
+		
+		return 0;
+	}
+	
 	sub _ntriples_string {
 		my $self	= shift;
 		my $str		= sprintf('"%s"', $self->__ntriples_string);
@@ -230,7 +290,7 @@ package Attean::API::Literal 0.007 {
 	};
 }
 
-package Attean::API::DateTimeLiteral 0.007 {
+package Attean::API::DateTimeLiteral 0.012 {
 	use Moo::Role;
 	use DateTime::Format::W3CDTF;
 	use namespace::clean;
@@ -241,12 +301,12 @@ package Attean::API::DateTimeLiteral 0.007 {
 	}
 }
 
-package Attean::API::CanonicalizingLiteral 0.007 {
+package Attean::API::CanonicalizingLiteral 0.012 {
 	use Moo::Role;
 	requires 'canonicalized_term';
 }
 
-package Attean::API::NumericLiteral 0.007 {
+package Attean::API::NumericLiteral 0.012 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed looks_like_number);
 	use namespace::clean;
@@ -437,14 +497,24 @@ package Attean::API::NumericLiteral 0.007 {
 	with 'Attean::API::Literal', 'Attean::API::CanonicalizingLiteral';
 }
 
-package Attean::API::Blank 0.007 {
+package Attean::API::Blank 0.012 {
 	use Moo::Role;
 	use Scalar::Util qw(blessed);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use Attean::API::Query;
 	use namespace::clean;
 	
 	sub ebv { return 1; }
 	with 'Attean::API::Term', 'Attean::API::BlankOrIRI';
-
+	with 'Attean::API::SPARQLSerializable';
+	
+	sub sparql_tokens {
+		my $self	= shift;
+		my $t	= AtteanX::SPARQL::Token->fast_constructor( BNODE, -1, -1, -1, -1, [$self->value] );
+		return Attean::ListIterator->new( values => [$t], item_type => 'AtteanX::SPARQL::Token' );
+	}
+	
 	sub compare {
 		my ($a, $b)	= @_;
 		return 1 unless blessed($b);
@@ -453,14 +523,29 @@ package Attean::API::Blank 0.007 {
 	}
 }
 
-package Attean::API::IRI 0.007 {
+package Attean::API::IRI 0.012 {
 	use Moo::Role;
 	use IRI;
 	use Scalar::Util qw(blessed);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
+	use Attean::API::Query;
 	use namespace::clean;
 	
 	sub ebv { return 1; }
 	with 'Attean::API::Term', 'Attean::API::BlankOrIRI';
+	with 'Attean::API::SPARQLSerializable';
+	
+	sub sparql_tokens {
+		my $self	= shift;
+		my @tokens;
+		if ($self->value eq '') {
+			push(@tokens, AtteanX::Parser::Turtle::Token->a);
+		} else {
+			push(@tokens, AtteanX::SPARQL::Token->fast_constructor( IRI, -1, -1, -1, -1, [$self->value] ));
+		}
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 	
 	sub compare {
 		my ($a, $b)	= @_;
@@ -491,6 +576,8 @@ package Attean::API::IRI 0.007 {
 
 __END__
 
+=back
+
 =head1 BUGS
 
 Please report any bugs or feature requests to through the GitHub web interface
@@ -506,7 +593,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
