@@ -7,7 +7,7 @@ Attean::CodeIterator - Iterator implementation backed by a generator function
 
 =head1 VERSION
 
-This document describes Attean::CodeIterator version 0.007
+This document describes Attean::CodeIterator version 0.012
 
 =head1 SYNOPSIS
 
@@ -52,16 +52,17 @@ that will be returned from the iterator.
 
 =cut
 
-package Attean::CodeIterator 0.007 {
+package Attean::CodeIterator 0.012 {
 	use Moo;
 	use Type::Tiny::Role;
+	use Scalar::Util qw(blessed);
 	use Types::Standard qw(CodeRef ArrayRef);
 	use namespace::clean;
 	
 	with 'Attean::API::Iterator';
 	
 	has generator => (is => 'ro', isa => CodeRef, required => 1);
-	has buffer => (is => 'ro', isa => ArrayRef, init_arg => undef, default => sub { [] });
+	has _buffer => (is => 'ro', isa => ArrayRef, init_arg => undef, default => sub { [] });
 
 =item C<< next >>
 
@@ -71,7 +72,7 @@ Returns the iterator's next item, or undef upon reaching the end of iteration.
 
 	sub next {
 		my $self	= shift;
-		my $buffer	= $self->buffer;
+		my $buffer	= $self->_buffer;
 		if (scalar(@$buffer)) {
 			return shift(@$buffer);
 		}
@@ -83,7 +84,7 @@ Returns the iterator's next item, or undef upon reaching the end of iteration.
 		}
 		my $role	= $self->item_type;
 		if (Role::Tiny->is_role($role)) {
-			die "CodeIterator item is not a $role" unless ($item->does($role));
+			die "CodeIterator item is not a $role" unless (blessed($item) and $item->does($role));
 		}
 		return $item;
 	}
@@ -110,7 +111,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
