@@ -7,7 +7,7 @@ Attean::Expression - SPARQL Expressions
 
 =head1 VERSION
 
-This document describes Attean::Expression version 0.007
+This document describes Attean::Expression version 0.012
 
 =head1 SYNOPSIS
 
@@ -40,19 +40,26 @@ use Attean::API::Expression;
 
 =cut
 
-package Attean::ValueExpression 0.007 {
+package Attean::ValueExpression 0.012 {
 	use Moo;
 	use Types::Standard qw(ConsumerOf);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use namespace::clean;
 
+	with 'Attean::API::SPARQLSerializable';
 	with 'Attean::API::Expression';
+
+	has 'value' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable']);
+
 	sub arity { return 0 }
 	sub BUILDARGS {
 		my $class	= shift;
 		return $class->SUPER::BUILDARGS(@_, operator => '_value');
 	}
-	has 'value' => (is => 'ro', isa => ConsumerOf['Attean::API::TermOrVariable']);
 	
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		return 1;
 	}
@@ -76,9 +83,17 @@ package Attean::ValueExpression 0.007 {
 		return;
 	}
 	
-	sub as_sparql {
+	sub sparql_tokens {
 		my $self	= shift;
-		return $self->value->as_sparql;
+		return $self->value->sparql_tokens;
+	}
+
+	sub unaggregated_variables {
+		my $self	= shift;
+		if ($self->value->does('Attean::API::Variable')) {
+			return $self->value;
+		}
+		return;
 	}
 }
 
@@ -86,11 +101,13 @@ package Attean::ValueExpression 0.007 {
 
 =cut
 
-package Attean::UnaryExpression 0.007 {
+package Attean::UnaryExpression 0.012 {
 	use Moo;
 	use Types::Standard qw(Enum);
 	use namespace::clean;
 
+	with 'Attean::API::UnaryExpression', 'Attean::API::Expression', 'Attean::API::UnaryQueryTree';
+
 	my %map	= ('NOT' => '!');
 	around 'BUILDARGS' => sub {
 		my $orig	= shift;
@@ -106,6 +123,8 @@ package Attean::UnaryExpression 0.007 {
 		$type->assert_valid($self->operator);
 	}
 	
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		my $self	= shift;
 		foreach my $c (@{ $self->children }) {
@@ -113,25 +132,27 @@ package Attean::UnaryExpression 0.007 {
 		}
 		return 1;
 	}
-	
-	with 'Attean::API::UnaryExpression', 'Attean::API::Expression', 'Attean::API::UnaryQueryTree';
 }
 
 =item * L<Attean::BinaryExpression>
 
 =cut
 
-package Attean::BinaryExpression 0.007 {
+package Attean::BinaryExpression 0.012 {
 	use Moo;
 	use Types::Standard qw(Enum);
 	use namespace::clean;
 
+	with 'Attean::API::BinaryExpression';
+
 	sub BUILD {
 		my $self	= shift;
 		state $type	= Enum[qw(+ - * / < <= > >= != = && ||)];
 		$type->assert_valid($self->operator);
 	}
 	
+	sub tree_attributes { return qw(operator) }
+
 	sub is_stable {
 		my $self	= shift;
 		foreach my $c (@{ $self->children }) {
@@ -139,20 +160,27 @@ package Attean::BinaryExpression 0.007 {
 		}
 		return 1;
 	}
-	
-	with 'Attean::API::BinaryExpression';
 }
 
 =item * L<Attean::FunctionExpression>
 
 =cut
 
-package Attean::FunctionExpression 0.007 {
+package Attean::FunctionExpression 0.012 {
 	use Moo;
 	use Types::Standard qw(Enum ConsumerOf HashRef);
 	use Types::Common::String qw(UpperCaseStr);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use namespace::clean;
 
+	has 'operator'		=> (is => 'ro', isa => UpperCaseStr, coerce => UpperCaseStr->coercion, required => 1);
+	has 'base'			=> (is => 'rw', isa => ConsumerOf['Attean::IRI'], predicate => 'has_base');
+
+	with 'Attean::API::NaryExpression';
+	with 'Attean::API::SPARQLSerializable';
+
+
 	around 'BUILDARGS' => sub {
 		my $orig	= shift;
 		my $class	= shift;
@@ -162,25 +190,11 @@ package Attean::FunctionExpression 0.007 {
 	};
 	sub BUILD {
 		my $self	= shift;
-		state $type	= Enum[qw(IN NOTIN STR LANG LANGMATCHES DATATYPE BOUND IRI URI BNODE RAND ABS CEIL FLOOR ROUND CONCAT SUBSTR STRLEN REPLACE UCASE LCASE ENCODE_FOR_URI CONTAINS STRSTARTS STRENDS STRBEFORE STRAFTER YEAR MONTH DAY HOURS MINUTES SECONDS TIMEZONE TZ NOW UUID STRUUID MD5 SHA1 SHA256 SHA384 SHA512 COALESCE IF STRLANG STRDT SAMETERM ISIRI ISURI ISBLANK ISLITERAL ISNUMERIC REGEX)];
+		state $type	= Enum[qw(INVOKE IN NOTIN STR LANG LANGMATCHES DATATYPE BOUND IRI URI BNODE RAND ABS CEIL FLOOR ROUND CONCAT SUBSTR STRLEN REPLACE UCASE LCASE ENCODE_FOR_URI CONTAINS STRSTARTS STRENDS STRBEFORE STRAFTER YEAR MONTH DAY HOURS MINUTES SECONDS TIMEZONE TZ NOW UUID STRUUID MD5 SHA1 SHA256 SHA384 SHA512 COALESCE IF STRLANG STRDT SAMETERM ISIRI ISURI ISBLANK ISLITERAL ISNUMERIC REGEX)];
 		$type->assert_valid($self->operator);
 	}
-	has 'operator'		=> (is => 'ro', isa => UpperCaseStr, coerce => UpperCaseStr->coercion, required => 1);
-	has 'base'			=> (is => 'rw', isa => ConsumerOf['Attean::IRI'], predicate => 'has_base');
-	with 'Attean::API::NaryExpression';
-	sub as_sparql {
-		my $self	= shift;
-		my $op		= $self->operator;
-		my @args	= @{ $self->children };
-		if ($op =~ /^(NOT)?IN$/) {
-			$op		=~ s/NOT/NOT /;
-			my ($t, @values)	= map { $_->as_sparql } @args;
-			return sprintf("%s %s (%s)", $t, $op, join(', ', @values));
-		} else {
-			my @values	= map { $_->as_sparql } @args;
-			return sprintf("%s(%s)", $op, join(', ', @values));
-		}
-	}
+
+	sub tree_attributes { return qw(operator) }
 
 	sub is_stable {
 		my $self	= shift;
@@ -190,12 +204,34 @@ package Attean::FunctionExpression 0.007 {
 		}
 		return 1;
 	}
+	
+	sub sparql_tokens {
+		my $self	= shift;
+		my $func	= AtteanX::SPARQL::Token->keyword($self->operator);
+		my $lparen	= AtteanX::SPARQL::Token->lparen;
+		my $rparen	= AtteanX::SPARQL::Token->rparen;
+		my $comma	= AtteanX::SPARQL::Token->comma;
+
+		my @tokens;
+		push(@tokens, $func, $lparen);
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $comma);
+		}
+		if (scalar(@tokens) > 2) {
+			pop(@tokens);	# remove the last comma
+		}
+		push(@tokens, $rparen);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
-package Attean::AggregateExpression 0.007 {
+package Attean::AggregateExpression 0.012 {
 	use Moo;
 	use Types::Standard qw(Bool Enum Str HashRef ConsumerOf);
 	use Types::Common::String qw(UpperCaseStr);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use namespace::clean;
 
 	around 'BUILDARGS' => sub {
@@ -206,21 +242,68 @@ package Attean::AggregateExpression 0.007 {
 		return $args;
 	};
 	sub BUILD {
-		state $type	= Enum[qw(COUNT SUM MIN MAX AVG GROUP_CONCAT SAMPLE)];
+		state $type	= Enum[qw(COUNT SUM MIN MAX AVG GROUP_CONCAT SAMPLE RANK)];
 		$type->assert_valid(shift->operator);
 	}
 	has 'operator'		=> (is => 'ro', isa => UpperCaseStr, coerce => UpperCaseStr->coercion, required => 1);
-	has 'scalar_vars'	=> (is => 'ro', isa => HashRef[Str], default => sub { +{} });
+	has 'scalar_vars'	=> (is => 'ro', isa => HashRef, default => sub { +{} });
 	has 'distinct'		=> (is => 'ro', isa => Bool, default => 0);
 	has 'variable'		=> (is => 'ro', isa => ConsumerOf['Attean::API::Variable'], required => 1);
+
 	with 'Attean::API::AggregateExpression';
+	with 'Attean::API::SPARQLSerializable';
+
+	sub tree_attributes { return qw(operator scalar_vars variable) }
+
+	sub is_stable {
+		my $self	= shift;
+		foreach my $expr (@{ $self->groups }, values %{ $self->aggregates }) {
+			return 0 unless ($expr->is_stable);
+		}
+		return 1;
+	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $distinct	= AtteanX::SPARQL::Token->keyword('DISTINCT');
+		my $func	= AtteanX::SPARQL::Token->keyword($self->operator);
+		my $lparen	= AtteanX::SPARQL::Token->lparen;
+		my $rparen	= AtteanX::SPARQL::Token->rparen;
+		my $comma	= AtteanX::SPARQL::Token->comma;
+
+		my @tokens;
+		push(@tokens, $func);
+		push(@tokens, $lparen);
+		if ($self->distinct) {
+			push(@tokens, $distinct);
+		}
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $comma);
+		}
+		if (scalar(@tokens) > 2) {
+			pop(@tokens);	# remove the last comma
+		}
+		my $vars	= $self->scalar_vars;
+		my @keys	= keys %$vars;
+		if (scalar(@keys)) {
+			die "TODO: Implement SPARQL serialization for aggregate scalar vars";
+		}
+		push(@tokens, $rparen);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
-package Attean::CastExpression 0.007 {
+package Attean::CastExpression 0.012 {
 	use Moo;
 	use Types::Standard qw(Enum ConsumerOf);
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use namespace::clean;
 
+	with 'Attean::API::SPARQLSerializable';
+	with 'Attean::API::UnaryExpression', 'Attean::API::Expression', 'Attean::API::UnaryQueryTree';
+
 	has 'datatype'	=> (is => 'ro', isa => ConsumerOf['Attean::API::IRI']);
 	sub BUILDARGS {
 		my $class	= shift;
@@ -232,6 +315,8 @@ package Attean::CastExpression 0.007 {
 		$type->assert_valid($self->datatype->value);
 	}
 	
+	sub tree_attributes { return qw(operator datatype) }
+
 	sub is_stable {
 		my $self	= shift;
 		foreach my $c (@{ $self->children }) {
@@ -240,26 +325,95 @@ package Attean::CastExpression 0.007 {
 		return 1;
 	}
 
-	with 'Attean::API::UnaryExpression', 'Attean::API::Expression', 'Attean::API::UnaryQueryTree';
+	sub sparql_tokens {
+		my $self	= shift;
+		my $dt		= AtteanX::SPARQL::Token->fast_constructor( IRI, -1, -1, -1, -1, [$self->datatype->value] ),
+		my $lparen	= AtteanX::SPARQL::Token->lparen;
+		my $rparen	= AtteanX::SPARQL::Token->rparen;
+		my $comma	= AtteanX::SPARQL::Token->comma;
+
+		my @tokens;
+		push(@tokens, $dt, $lparen);
+		foreach my $t (@{ $self->children }) {
+			push(@tokens, $t->sparql_tokens->elements);
+			push(@tokens, $comma);
+		}
+		if (scalar(@tokens) > 2) {
+			pop(@tokens);	# remove the last comma
+		}
+		push(@tokens, $rparen);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
 }
 
-package Attean::ExistsExpression 0.007 {
+package Attean::ExistsExpression 0.012 {
 	use Moo;
+	use AtteanX::SPARQL::Constants;
+	use AtteanX::SPARQL::Token;
 	use Types::Standard qw(ConsumerOf);
 	use namespace::clean;
 
+	with 'Attean::API::SPARQLSerializable';
 	with 'Attean::API::Expression';
+
 	sub arity { return 0 }
 	sub BUILDARGS {
 		my $class	= shift;
 		return $class->SUPER::BUILDARGS(@_, operator => '_exists');
 	}
 	has 'pattern' => (is => 'ro', isa => ConsumerOf['Attean::API::Algebra']);
+	sub as_string {
+		my $self	= shift;
+		my $sparql	= $self->pattern->as_sparql;
+		$sparql		=~ s/\s+/ /g;
+		return "EXISTS { $sparql }";
+	}
+
+	sub tree_attributes { return qw(operator pattern) }
+
+	sub is_stable {
+		my $self	= shift;
+		# TODO: need deep analysis of exists pattern to tell if this is stable
+		# (there might be an unstable filter expression deep inside the pattern)
+		return 0;
+	}
+
+	sub sparql_tokens {
+		my $self	= shift;
+		my $exists	= AtteanX::SPARQL::Token->keyword('EXISTS');
+		my $lbrace	= AtteanX::SPARQL::Token->lbrace;
+		my $rbrace	= AtteanX::SPARQL::Token->rbrace;
+		my $child	= $self->pattern;
+		
+		my @tokens;
+		push(@tokens, $exists, $lbrace);
+		push(@tokens, $child->sparql_tokens->elements);
+		push(@tokens, $rbrace);
+		return Attean::ListIterator->new( values => \@tokens, item_type => 'AtteanX::SPARQL::Token' );
+	}
+
+	sub unaggregated_variables {
+		my $self	= shift;
+		return map { Attean::Variable->new($_) } $self->pattern->in_scope_variables;
+	}
+}
+
+package Attean::ExistsPlanExpression 0.012 {
+	use Moo;
+	use Types::Standard qw(ConsumerOf);
+	use namespace::clean;
+
+	with 'Attean::API::Expression';
+	sub arity { return 0 }
+	sub BUILDARGS {
+		my $class	= shift;
+		return $class->SUPER::BUILDARGS(@_, operator => '_existsplan');
+	}
+	has 'plan' => (is => 'ro', isa => ConsumerOf['Attean::API::BindingSubstitutionPlan']);
 	sub as_string {
 		my $self	= shift;
 		# TODO: implement as_string for EXISTS patterns
-		warn "TODO: Attean::ExistsExpression->as_string";
-		return "EXISTS { ... }";
+		return "Attean::ExistsPlanExpression { ... }";
 	}
 	sub as_sparql {
 		my $self	= shift;
@@ -269,16 +423,22 @@ package Attean::ExistsExpression 0.007 {
 		my $indent	= $sp x $level;
 
 		# TODO: implement as_string for EXISTS patterns
-		warn "TODO: Attean::ExistsExpression->as_string";
-		return "EXISTS " . $self->pattern->as_sparql( level => $level+1, indent => $sp );
+		return "EXISTS { " . $self->pattern->as_sparql( level => $level+1, indent => $sp ) . " }";
 	}
 
+	sub tree_attributes { return qw(operator plan) }
+
 	sub is_stable {
 		my $self	= shift;
 		# TODO: need deep analysis of exists pattern to tell if this is stable
 		# (there might be an unstable filter expression deep inside the pattern)
 		return 0;
 	}
+
+	sub unaggregated_variables {
+		my $self	= shift;
+		die "unaggregated_variables cannot be called on Attean::ExistsPlanExpression";
+	}
 }
 
 
@@ -303,7 +463,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2014 Gregory Todd Williams.
+Copyright (c) 2014--2016 Gregory Todd Williams.
 This program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
