@@ -7,7 +7,7 @@ AtteanX::Parser::Turtle::Constants - Constant definitions for use in parsing Tur
 
 =head1 VERSION
 
-This document describes AtteanX::Parser::Turtle::Constants version 0.007
+This document describes AtteanX::Parser::Turtle::Constants version 0.012
 
 =head1 SYNOPSIS
 
@@ -19,45 +19,43 @@ This document describes AtteanX::Parser::Turtle::Constants version 0.007
 
 =cut
 
-package AtteanX::Parser::Turtle::Constants 0.007 {
+package AtteanX::Parser::Turtle::Constants 0.012 {
 	use v5.14;
 	use warnings;
 
-	our $VERSION;
 	our @EXPORT;
 	BEGIN {
-		$VERSION				= 0.007;
 		@EXPORT = qw(
-			LBRACKET
-			RBRACKET
-			LPAREN
-			RPAREN
-			DOT
-			SEMICOLON
-			COMMA
-			HATHAT
 			A
-			BOOLEAN
-			PREFIXNAME
-			IRI
-			BNODE
-			DOUBLE
-			DECIMAL
-			INTEGER
-			WS
-			COMMENT
-			STRING3D
-			STRING3S
-			STRING1D
-			STRING1S
 			BASE
-			PREFIX
-			SPARQLBASE
-			SPARQLPREFIX
+			BNODE
+			BOOLEAN
+			COMMA
+			COMMENT
+			DECIMAL
+			DOT
+			DOUBLE
+			EQUALS
+			HATHAT
+			INTEGER
+			IRI
 			LANG
 			LBRACE
+			LBRACKET
+			LPAREN
+			PREFIX
+			PREFIXNAME
 			RBRACE
-			EQUALS
+			RBRACKET
+			RPAREN
+			SEMICOLON
+			STRING1D
+			STRING1S
+			STRING3D
+			STRING3S
+			TURTLEBASE
+			TURTLEPREFIX
+			WS
 			decrypt_constant
 		)
 	};
@@ -103,7 +101,7 @@ Toby Inkster C<< <tobyink@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2012 Toby Inkster. This
+Copyright (c) 2014--2016 Toby Inkster. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
