@@ -4,7 +4,7 @@ AtteanX::Serializer::CanonicalNTriples - Canonical N-Triples Serializer
 
 =head1 VERSION
 
-This document describes AtteanX::Serializer::CanonicalNTriples version 0.007
+This document describes AtteanX::Serializer::CanonicalNTriples version 0.012
 
 =head1 SYNOPSIS
 
@@ -25,7 +25,7 @@ This document describes AtteanX::Serializer::CanonicalNTriples version 0.007
 use v5.14;
 use warnings;
 
-package AtteanX::Serializer::CanonicalNTriples 0.007 {
+package AtteanX::Serializer::CanonicalNTriples 0.012 {
 	use Moo;
 	use Encode;
 	extends 'AtteanX::Serializer::NTriples';
@@ -42,8 +42,13 @@ L<IO::Handle> object C<< $fh >>.
 		my $self	= shift;
 		my $io		= shift;
 		my $iter	= shift;
-		my $bytes	= $self->serialize_iter_to_bytes($iter);
-		$io->print(decode('UTF-8', $bytes, Encode::FB_CROAK));
+		$iter		= $iter->materialize;
+		my $triples	= $iter->canonical_set();
+		
+		foreach my $t (@$triples) {
+			my $str = $t->tuples_string;
+			$io->print($str . "\n");
+		}
 		return;
 	}
 	
@@ -57,15 +62,11 @@ and returns the serialization as a UTF-8 encoded byte string.
 	sub serialize_iter_to_bytes {
 		my $self	= shift;
 		my $iter	= shift;
-		$iter		= $iter->materialize;
-		my $triples	= $iter->canonical_set();
-		
-		my $data	= '';
-		foreach my $t (@$triples) {
-			my $str = $t->tuples_string;
-			$data	.= $str . "\n";
-		}
-		return encode('UTF-8', $data);
+		my $data	= encode('UTF-8', '');
+		open(my $fh, '>', \$data);
+		$self->serialize_iter_to_io($fh, $iter);
+		close($fh);
+		return $data;
 	}
 }
 
@@ -86,7 +87,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
+Copyright (c) 2014--2016 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
