@@ -4,7 +4,7 @@ AtteanX::Store::Memory - Simple in-memory RDF store
 
 =head1 VERSION
 
-This document describes AtteanX::Store::Memory version 0.002
+This document describes AtteanX::Store::Memory version 0.012
 
 =head1 SYNOPSIS
 
@@ -19,7 +19,7 @@ AtteanX::Store::Memory provides an in-memory quad-store.
 use v5.14;
 use warnings;
 
-package AtteanX::Store::Memory 0.001 {
+package AtteanX::Store::Memory 0.012 {
 use Moo;
 use Type::Tiny::Role;
 use Types::Standard qw(Int ArrayRef HashRef ConsumerOf InstanceOf);
@@ -28,13 +28,31 @@ use Set::Scalar;
 use Digest::SHA;
 use List::Util qw(first);
 use Scalar::Util qw(refaddr reftype blessed);
+use Math::Cartesian::Product;
 use namespace::clean;
 
 with 'Attean::API::MutableQuadStore';
 with 'Attean::API::QuadStore';
+with 'Attean::API::ETagCacheableQuadStore';
+with 'Attean::API::TimeCacheableQuadStore';
+with 'Attean::API::CostPlanner';
 
 my @pos_names	= Attean::API::Quad->variables;
 
+=head1 ATTRIBUTES
+
+=over 4
+
+=item C<< subject >>
+
+=item C<< predicate >>
+
+=item C<< object >>
+
+=item C<< graph >>
+
+=back
+
 =head1 METHODS
 
 Beyond the methods documented below, this class inherits methods from the
@@ -56,6 +74,7 @@ has object => (is => 'ro', isa => HashRef[InstanceOf['Set::Scalar']], init_arg =
 has graph => (is => 'ro', isa => HashRef[InstanceOf['Set::Scalar']], init_arg => undef, default => sub { +{} });
 has graph_nodes	=> (is => 'rw', isa => HashRef[ConsumerOf['Attean::API::IRI']], init_arg => undef, default => sub { +{} });
 has hash		=> (is => 'rw', isa => InstanceOf['Digest::SHA'], default => sub { Digest::SHA->new });
+has mtime		=> (is => 'rw', isa => Int, default => sub { return time() });
 
 =item C<< size >>
 
@@ -75,6 +94,14 @@ predicate and objects. Any of the arguments may be undef to match any value.
 =cut
 
 sub get_quads {
+	my $self	= shift;
+	my @nodes	= map { ref($_) eq 'ARRAY' ? $_ : [$_] } @_;
+	my @iters;
+	cartesian { push(@iters, $self->_get_quads(@_)) } @nodes;
+	return Attean::IteratorSequence->new( iterators => \@iters, item_type => 'Attean::API::Quad' );
+}
+
+sub _get_quads {
 	my $self	= shift;
 	my @nodes	= @_;
 	my $bound	= 0;
@@ -184,6 +211,7 @@ sub add_quad {
 		$self->_size($self->_size + 1);
 		my $id	= scalar(@{ $self->statements });
 		$self->hash->add('+' . encode_utf8($st->as_string));
+		$self->mtime(time());
 		push( @{ $self->statements }, $st );
 		foreach my $pos (0 .. $#pos_names) {
 			my $name	= $pos_names[ $pos ];
@@ -222,6 +250,7 @@ sub remove_quad {
 		$self->_size( $self->_size - 1 );
 		my $id	= $self->_statement_id( $st->values );
 		$self->hash->add('-' . encode_utf8($st->as_string));
+		$self->mtime(time());
 		$self->statements->[ $id ]	= undef;
 		foreach my $pos (0 .. 3) {
 			my $name	= $pos_names[ $pos ];
@@ -247,6 +276,13 @@ Removes the specified C<$statement> from the underlying model.
 =cut
 
 sub remove_quads {
+	my $self	= shift;
+	my @nodes	= map { ref($_) eq 'ARRAY' ? $_ : [$_] } @_;
+	my @iters;
+	cartesian { $self->_remove_quads(@_) } @nodes;
+}
+
+sub _remove_quads {
 	my $self	= shift;
 	my $subj	= shift;
 	my $pred	= shift;
@@ -307,6 +343,14 @@ value.
 =cut
 
 sub count_quads {
+	my $self	= shift;
+	my @nodes	= map { ref($_) eq 'ARRAY' ? $_ : [$_] } @_;
+	my $count	= 0;
+	cartesian { $count += $self->_count_quads(@_) } @nodes;
+	return $count;
+}
+
+sub _count_quads {
 	my $self	= shift;
 	my @nodes	= @_[0..3];
 	my $bound	= 0;
@@ -374,6 +418,15 @@ sub etag_value_for_quads {
 	return $self->hash->b64digest;
 }
 
+=item C<< mtime_for_quads >>
+
+=cut
+
+sub mtime_for_quads {
+	my $self	= shift;
+	return $self->mtime;
+}
+
 sub _statement_id {
 	my $self	= shift;
 	my @nodes	= @_;
@@ -403,13 +456,49 @@ sub _statement_id {
 		$i	= $i->intersection($s);
 	}
 	if ($i->size == 1) {
-		my ($id)	= $i->members;
+		my ($id)	= $i->elements;
 		return $id;
 	} else {
 		return -1;
 	}
 }
 
+
+=item C<< plans_for_algebra >>
+
+The store implements a cost-based query planner, but this method is
+reimplemented to hand the overall control of the planning process to
+an external planner by returning C<undef>.
+
+=cut
+
+sub plans_for_algebra {
+	my $self	= shift;
+	my $algebra	= shift;
+	return;
+}
+
+
+=item C<< cost_for_plan >>
+
+This store provides a cost estimate only for retrieving individual
+quad patterns in this method. It will allow other planners to estimate
+the cost for any other parts of the plan by returning C<undef> for
+those parts.
+
+=cut
+
+sub cost_for_plan {
+	my $self	= shift;
+	my $plan	= shift;
+	if ($plan->isa('Attean::Plan::Quad')) {
+		my @values	= $plan->values;
+		my $count	= $self->count_quads(@values);
+		return $count;
+	}
+	return;
+}
+
 }
 
 1;
@@ -429,7 +518,7 @@ Gregory Todd Williams  C<< <gwilliams@cpan.org> >>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2012 Gregory Todd Williams. This
+Copyright (c) 2014--2016 Gregory Todd Williams. This
 program is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
